/*
 * Distributed under GPLv3
 * Copyright (C) 2011 Zhangxiaowen Gong
 * All right reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * * Neither the name of the copyright holders nor the names of
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file DataBlock.cpp
 * \brief Source file for the data block base class
 *
 * \author Zhangxiaowen Gong
 * \date 2011/3/19
 */

#include "DataBlock.hpp"

CDataControl *CDataControl::m_instance = NULL;

uint32_t CDataControl::wait_for_free(CData *data) {
	bool int_sta = Is_global_interrupt_enabled();
	Disable_global_interrupt();
	m_tail->m_next = data;
	m_tail = data;
	data->m_next = NULL;
	m_len++;
	if (int_sta)
		Enable_global_interrupt();
	return m_len;
}

void CDataControl::free_all(void) {
	while (m_len) {
		// m_ucCount is volatile, so more CData object can be added
		// into the list while executing this function
		CData* tmp = m_head;
		m_head = tmp->m_next;
		if (tmp->m_sub_buffer)
			delete[] (tmp->m_sub_buffer); // use delete [] to free array
		delete tmp;
		m_len--;
	}
}

