/*
 * TWI_ASYN.hpp
 *
 *  Created on: Apr 9, 2012
 *      Author: Andygongyb
 */

#ifndef TWI_HPP_
#define TWI_HPP_

#include <avr32/io.h>
#include <stdint.h>
#include <cstddef>
#include "FreeRTOS/FreeRTOS.h"
#include "FreeRTOS/portmacro.h"
#include "FreeRTOS/task.h"
#include "FreeRTOS/semphr.h"

extern "C" {

#include "intc.h"
#include "gpio.h"

}

#include "DataBlock.hpp"
#include "Initialization.hpp"
#include "PDCA.hpp"

extern "C" void twim0_pdca_irq(void) __attribute__((__interrupt__, naked));
extern "C" void twim1_pdca_irq(void) __attribute__((__interrupt__, naked));
extern "C" void twim0_irq(void) __attribute__((__interrupt__, naked));
extern "C" void twim1_irq(void) __attribute__((__interrupt__, naked));

typedef class CTWIM: public CPDCA_Peripheral {

public:
	typedef enum {
		WRITE = 0, READ = 1
	} mode_t;

	typedef enum {
		TWIM0 = 0, TWIM1 = 1,
	} module_t;

	typedef struct {
		mode_t mode :1;
		uint8_t slave_addr :7;
		uint8_t* buffer;
		uint8_t buffer_len;
		call_back_t before;
		call_back_t after;
		volatile flag_t* if_done;
		xTaskHandle task;
	} packet_t;

protected:

	volatile avr32_twim_t* m_twim;
	void (*m_handler)(void);
	bool m_if_busy;
	CPDCA::module_t m_tx_module;
	packet_t* m_read_pack;

	CTWIM(module_t module);

	typedef class CCMD: public CData {
	private:
	public:
		packet_t m_packet;
	public:
		CCMD(packet_t* packet = NULL);
		friend class CTWIM;
		friend void twi_irq(void);
	}*pCCMD;

	CCMD *m_head;
	CCMD *m_tail;
	volatile uint32_t m_len;
	
	xSemaphoreHandle m_mutex;

	void start(packet_t* packet) __attribute__((always_inline));

	portBASE_TYPE next_cmd(flag_t if_done) __attribute__((always_inline));

public:

	void set_freq(uint32_t clk);

	virtual ~CTWIM(void) {
		CCMD *ptr1 = m_head, *ptr2 = NULL;
		CDataControl *data_control = CDataControl::instance();
		m_twim->CR.mdis = 1;
		while (ptr2 != m_tail) { // free the queue
			ptr2 = ptr1;
			ptr1 = static_cast<CCMD *> (ptr1->m_next);
			if (ptr2->m_packet.if_done)
				*(ptr2->m_packet.if_done) = FAILED;
			data_control->wait_for_free(ptr2);
		}
		m_if_busy = false;
	}

	void append(packet_t* packet, bool need_free = false, bool need_block = false);

	bool if_busy(void) {
		return m_if_busy;
	}
	uint32_t get_queue_len(void) {
		return m_len;
	}

	friend void twim0_pdca_irq(void);
	friend void twim1_pdca_irq(void);
	friend void twim0_irq(void);
	friend void twim1_irq(void);
}*pCTWIM;

template<CTWIM::module_t N>
class CTWIM_ch: public CTWIM {
private:
	static CTWIM_ch<N>* m_instance;
	CTWIM_ch(void) :
		CTWIM(N) {
		//
	}

public:
	static CTWIM_ch<N>* instance(void) {
		if (!m_instance) {
			m_instance = new CTWIM_ch<N> ();
		}
		return m_instance;
	}

	~CTWIM_ch(void) {
		m_instance = NULL;
	}
};

typedef class CTWIM_device {
protected:
	CTWIM* const m_twim;
	const uint8_t m_addr;
	CTWIM_device(CTWIM* the_twim, uint8_t addr)
		: m_twim(the_twim), m_addr(addr) {
		//
	}
public:
	void reg_w (uint8_t reg, uint8_t val);
	void write_seq (uint8_t* buffer, uint8_t len);
	void read_seq (uint8_t* buffer, uint8_t len);
	uint8_t reg_r (uint8_t reg);
} *pCTWIM_device;

#endif
